//OrderDistiller.java modified by Pietro Terna (look below at rows signed //pp)
//OrderDistiller.java modified by Marco Lamieri and Francesco Merlo (look below at rows signed //lm)
import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
import swarm.collections.ListImpl;

/**
 * This class is used to read data from two worksheets.<br />
 * The first one contains the list of recipes of our virtual enterprise.<br/>
 * The second one contains a sequence of orders to be launched,
 * shift by shift, in order to make the daily production activities.
 *
 * @author Cristian Barreca, Elena Bonessa, Antonella Borra. Modified by: Marco Lamieri,
 *  Francesco Merlo. Refactored by Francesco Merlo 
 *
 */
public class OrderDistiller extends OrderGenerator
{
	String
		semicolon = ";",
		gate = "#",
		layer="l",
		computation="c",
		orderSequences1 = "recipeData/orderStartingSequence.xls", //The first worksheet of orders
		orderSequences2 = "recipeData/orderSequence.xls", //The second worksheet of orders
		recipeFile = "recipeData/recipes.xls", //The worksheet of recipes
		currentOrderSequenceWorksheet;
	int
		orderCount = 0,
		currentLayer = 0;
	boolean
		firstTime = true,
		orderSequenceWorksheetOpen = false;
	int[]
		currentOrder;
	ExcelReader
		orderSequenceWorksheet,
		recipeWorksheet;
	AssigningTool
		assigningTool;//pt
	Recipe
		aRecipe;
	Unit
		aUnit;
	Order
		anOrder;
	ListImpl
		unitList,
		endUnitList,
		orderList,
		recipeList;

	// CONSTRUCTOR
	public OrderDistiller (Zone aZone, int msn, int msl, ListImpl ul,
		ListImpl eul, ListImpl ol, int tln, ESFrameModelSwarm mo, AssigningTool at)
	{ //pt
		super(aZone, msn, msl, ul, eul, ol, tln, mo, at);

		unitList=ul;
		endUnitList=eul;
		orderList=ol;
		assigningTool=at; //pt
	}

	/**This is the method containing the iterator needed to launch the daily production of recipes.<br />
	* It take a look at the orderSequence arrays to determine which recipes must be done and how many times. <br />
	* A request for which unit can do the first production phase of each recipe will be done to units or endUnits.
	*/
	public void distill()
	{
		if(firstTime == true)
		{
			currentOrderSequenceWorksheet = orderSequences1;

			firstTime = false;
		}
		else
			currentOrderSequenceWorksheet = orderSequences2;

		if(!orderSequenceWorksheetOpen)
		{
			orderSequenceWorksheet = new ExcelReader(currentOrderSequenceWorksheet);
			orderSequenceWorksheetOpen = true;
		}

		checkForComment(); //see below
		//Getting the number of the shift
		int shiftNumber = orderSequenceWorksheet.getIfInteger();

		boolean isEndOfShift = false;

		while(!isEndOfShift)
		{
			if(orderSequenceWorksheet.checkForLabelCell())
			{
				String aString = orderSequenceWorksheet.getStrValue();
				if(aString.equals(semicolon))
				{
					isEndOfShift = true;
					break;
				}
				else
				{
					orderSequenceWorksheet.goBack();
				}
			}

			//Reading the order from the worksheet
			readOrderFrom(orderSequenceWorksheet);//see below

			//Checking if the recipe cose exists
			boolean recipeCodeFound = false;
			for (int r = 0; r < recipeList.getCount() && !recipeCodeFound; r++)
			{
				aRecipe = (Recipe) recipeList.atOffset(r);
				if(aRecipe.getRecipeCode() == currentOrder[0])
					recipeCodeFound = true;
			}

			if(recipeCodeFound == false)
			{
				System.err.println("OrderDistiller error: There is no Receipe with code '" + currentOrder[0] +
						"' check '" + recipeFile + "' and '" + currentOrderSequenceWorksheet + "'");
				MyExit.exit(1);
			}

		 	for(int q = 0; q < currentOrder[1]; q++)
			{
				orderCount++;

				anOrder = new Order(getZone(), orderCount, Globals.env.getCurrentTime(),
					aRecipe.getLength(), aRecipe.getRecipeSteps(),eSFrameModelSwarm, endUnitList);

				anOrder.setRecipeName(aRecipe.getRecipeName());
				// setting the layer (from 0 to totalLayerNumber-1)
				if(currentLayer <0 || currentLayer >= totalLayerNumber)
				    {
					System.out.println(
					"A layer in the sequence is not inclued\n"+
					"in the interval from 0 to totalLayerNumber-1");
					MyExit.exit(1);
				    }
				anOrder.setOrderLayer(currentLayer);
				// add the active orders to the general order list (they will be
				// eliminated when dropped in a unit, being finished); this
				// list has been introduced for accounting purposes [may be it would
				// be better substitute it with a get to the units to know their
				// waiting lists]
				orderList.addLast(anOrder);
				// sending the order to the first production unit (we are acting as the Front End of the ES)
				assigningTool.assign(anOrder);

				if(StartESFrame.verbose)
				{
					System.out.println("OrderDistiller: Order #" + anOrder.getOrderNumber() +
						" with Name '" + anOrder.getRecipeName() +
						"' and layerNumber '" + anOrder.getOrderLayer() + "' is starting production");
				}
			}
		}
		if(orderSequenceWorksheet.eof())
			orderSequenceWorksheetOpen = false;
	}
	/**
	* This method is used to collect the names of the units and of the end units, so that it can operate the check of
	* corrispondency between the production phases required by recipes and the phases of production the units can do.
	*/
	public void setDictionary()
	{
		super.setDictionary();
		//The List of Recipes
		recipeList = new ListImpl(getZone());

		recipeWorksheet = new ExcelReader(recipeFile);

		while(! recipeWorksheet.eof())
		{
				aRecipe = new Recipe(getZone());
				aRecipe.setRecipeFrom(recipeWorksheet);
				recipeList.addLast(aRecipe);
		}

		if(StartESFrame.verbose)
		{
			System.out.println("OrderDistiller: " + recipeList.getCount() + "recipes readed.");
			for (int i = 0; i < recipeList.getCount(); i++)
			{
				aRecipe = (Recipe) recipeList.atOffset(i);
				System.out.println("Recipe #" + i + " with name '" + aRecipe.getRecipeName() +
					"' and code '" + aRecipe.getRecipeCode() +"'");
			}
		}
	}
	
	// *****************
	// PRIVATE FUNCTIONS
	// *****************
	private int[] readOrderFrom(ExcelReader e)
	{
		orderSequenceWorksheet = e;
		currentOrder = new int[2];

		checkForComment();
		checkForLayer();
		currentOrder[0] = orderSequenceWorksheet.getIfInteger();
		orderSequenceWorksheet.getIfString();
		currentOrder[1] = orderSequenceWorksheet.getIfInteger();

		return currentOrder;
	}

	private void  checkForLayer()
	{
		if(orderSequenceWorksheet.checkForLabelCell())
		{
			String isLayer = orderSequenceWorksheet.getStrValue();

			if(isLayer.equals(layer))
			{
				currentLayer = orderSequenceWorksheet.getIntValue();

				if(StartESFrame.verbose)
					System.out.println("OrderDistiller: Current Layer is #" + currentLayer);
			}
			else
			{
				orderSequenceWorksheet.goBack();
			}
		}
	}

	private void checkForComment()
	{
		if(orderSequenceWorksheet.checkForLabelCell())
	 	{
			String isGate = orderSequenceWorksheet.getStrValue();
			if(isGate.equals(gate))
			{
				if(StartESFrame.verbose)
				{
					System.out.println("OrderDistiller: there is a comment ...");
					System.out.print("# ");
				}
				while(!orderSequenceWorksheet.eol())
				{
					String comment = orderSequenceWorksheet.getStrValue();
					if(StartESFrame.verbose)
						System.out.print(" " + comment + " |");
				}
				String comment = orderSequenceWorksheet.getStrValue();
				if(StartESFrame.verbose)
					System.out.println(" " + comment + " #");
					
				//Checking for other lines of comments
				checkForComment();
			}
			else
			{
				orderSequenceWorksheet.goBack();
			}
		}
	}

} // OrderDistiller
